import { Http, IDictionary, IRequestConfig } from '@zeedhi/core';

interface RouteStructure {
    resource: string;
    uniqueKey: string;
    operation: 'create' | 'read' | 'update' | 'delete';
}

/**
 * Classe para realizar a conversão de requisições para o padrão REST
 */
export class HttpCRUDTranslator {
    // Mapeamento entre rotas do back - rotas do mirage
    private static routeMap: IDictionary<RouteStructure> = {
        '/alterarStatusItem': {
            resource: 'alterarStatusItem', uniqueKey: 'NRCHECKLISTAGEND', operation: 'create',
        },
        '/alterarStatusTarefa': {
            resource: 'alterarStatusTarefa', uniqueKey: 'NRCHECKLISTAGEND', operation: 'create',
        },
        '/anexarImagensTarefas': {
            resource: 'anexarImagensTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'create',
        },
        '/salvarObservacao': {
            resource: 'salvarObservacao', uniqueKey: 'NRCHECKLISTAGEND', operation: 'create',
        },
        '/finalizarTarefa': {
            resource: 'finalizarTarefa', uniqueKey: 'NRCHECKLISTAGEND', operation: 'create',
        },
        '/salvarCriterios': {
            resource: 'salvarCriterios', uniqueKey: 'NRGERENCIAMENTOCHK', operation: 'create',
        },
        '/finalizarVistoria': {
            resource: 'finalizarVistoria', uniqueKey: 'NRGERENCIAMENTOCHK', operation: 'create',
        },
        '/anexarImagensVistoria': {
            resource: 'anexarImagensVistoria', uniqueKey: 'NRGERENCIAMENTOCHK', operation: 'create',
        },
        '/excluirFotoVistorias': {
            resource: 'excluirFotoVistorias', uniqueKey: 'NRFOTOTAREFAGERCHK', operation: 'create',
        },
        '/salvarDadosObservacao': {
            resource: 'salvarDadosObservacao', uniqueKey: 'ARRAYOBS', operation: 'create',
        },
        '/inicarOuFinalizarAuditoria': {
            resource: 'inicarOuFinalizarAuditoria', uniqueKey: 'NRAUDITORIA', operation: 'create',
        },
        '/alterarStatusCriterio': {
            resource: 'alterarStatusCriterio', uniqueKey: 'criteriosS', operation: 'create',
        },
        '/anexarImagemAuditoria': {
            resource: 'anexarImagemAuditoria', uniqueKey: 'NRAUDITORIA', operation: 'create',
        },
        '/excluirImagemAuditoria': {
            resource: 'excluirImagemAuditoria', uniqueKey: 'IMG', operation: 'create',
        },
        '/criarSolicitacao': {
            resource: 'criarSolicitacao', uniqueKey: 'NMSOLICITACAO', operation: 'create',
        },
        '/avaliarSolicitacao': {
            resource: 'avaliarSolicitacao', uniqueKey: 'NRSOLICITACAO', operation: 'create',
        },
        '/excluirFotoTarefas': {
            resource: 'excluirFotoTarefas', uniqueKey: 'NRFOTOTAREFAAGEND', operation: 'create',
        },
        '/getTarefasCalendario': {
            resource: 'getTarefasCalendario', uniqueKey: 'CODIGO', operation: 'read',
        },
        '/getTarefasListadas': {
            resource: 'getTarefasListadas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/getVistoriasListadas': {
            resource: 'getVistoriasListadas', uniqueKey: 'NRGERENCIAMENTOCHK', operation: 'read',
        },
        '/getFotosVistorias': {
            resource: 'getFotosVistorias', uniqueKey: 'NRFOTOTAREFAGERCHK', operation: 'read',
        },
        '/getCriterios': {
            resource: 'getCriterios', uniqueKey: 'NRCRITERIOTARGERCHK', operation: 'read',
        },
        '/getImagensAuditoria': {
            resource: 'getImagensAuditoria', uniqueKey: 'NRFOTORESULTAUDIT', operation: 'read',
        },
        // '/excluirImagemAuditoria': {
        //     resource: 'excluirImagemAuditoria', uniqueKey: 'IMG', operation: 'create',
        // },
        // '/excluirImagemAuditoria': {
        //     resource: 'excluirImagemAuditoria', uniqueKey: 'IMG', operation: 'create',
        // },
        // '/excluirImagemAuditoria': {
        //     resource: 'excluirImagemAuditoria', uniqueKey: 'IMG', operation: 'create',
        // },
        // aqui podemos adicionar rotas de outros cruds
    };

    public static post(url: string, data?: any, config?: IRequestConfig): Promise<any> {
        // se estiver online não precisa realizar a conversão
        if (navigator.onLine) return Http.post(url, data, config);

        const structure = this.routeMap[url];
        const {
            resource, uniqueKey, operation,
        } = structure;

        // Trata operação de read, convertendo o filtro para query string
        if (operation === 'read') {
            const query = data ? data.reduce(
                (result: string, filter: any) => (`${result}${filter.name}=${filter.value}&`), '?',
            ).slice(0, -1) : '';
            return Http.get(`${resource}${query}`, config);
        }

        // Trata operação de create, realizando 1 post para cada linha criada
        if (operation === 'create') {
            if (!Array.isArray(data)) return Http.post(`${resource}`, data);

            const promises = data.map(async (row) => Http.post(resource, row));
            return Promise.all(promises);
        }

        // Trata operação de update, realizando 1 put para cada linha editada
        if (operation === 'update') {
            if (!Array.isArray(data)) return Http.put(`${resource}/${data[uniqueKey]}`, data);

            const promises = data.map(async (row) => Http.put(`${resource}/${row[uniqueKey]}`, row));
            return Promise.all(promises);
        }

        // Trata operação de delete, passando a chave única da linha na url
        if (operation === 'delete') {
            return Http.delete(`${resource}/${data[uniqueKey]}`, config);
        }

        // Se nenhuma operação for encontrada, apenas retorna null
        return new Promise(() => null);
    }
}
